@@ -35,6 +35,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 """
 
+import copy
 from itertools import groupby, count
 import logging
 import os
@@ -103,6 +104,7 @@ class SNPs:
         self._discrepant_XY = get_empty_snps_dataframe()
         self._heterozygous_MT = get_empty_snps_dataframe()
         self._discrepant_vcf_position = get_empty_snps_dataframe()
+        self._low_quality = get_empty_snps_dataframe().index
         self._discrepant_merge_positions = pd.DataFrame()
         self._discrepant_merge_genotypes = pd.DataFrame()
         self._source = []
@@ -112,6 +114,9 @@ class SNPs:
         self._output_dir = output_dir
         self._resources = Resources(resources_dir=resources_dir)
         self._parallelizer = Parallelizer(parallelize=parallelize, processes=processes)
+        self._cluster = ""
+        self._chip = ""
+        self._chip_version = ""
 
         if file:
 
@@ -218,6 +223,30 @@ class SNPs:
         """
         return self._snps
 
+    @property
+    def snps_qc(self):
+        """Normalized SNPs, after quality control.
+
+        Any low quality SNPs, identified per
+        :meth:`identify_low_quality_snps() <snps.snps.SNPs.identify_low_quality_snps>`,
+        are not included in the result.
+
+        Returns
+        -------
+        pandas.DataFrame
+            normalized ``snps`` dataframe
+        """
+        if len(self._low_quality) == 0:
+            # ensure low quality SNPs, if any, are identified
+            self.identify_low_quality_snps()
+
+        if len(self._low_quality) > 0:
+            # filter out low quality SNPs
+            return self._snps.drop(self._low_quality)
+        else:
+            # no low quality SNPs to filter
+            return self._snps
+
     @property
     def duplicate(self):
         """Duplicate SNPs.
@@ -268,6 +297,22 @@ class SNPs:
         """
         return self._discrepant_vcf_position
 
+    @property
+    def low_quality(self):
+        """SNPs identified as low quality, if any, per
+        :meth:`identify_low_quality_snps() <snps.snps.SNPs.identify_low_quality_snps>`.
+
+        Returns
+        -------
+        pandas.DataFrame
+            normalized ``snps`` dataframe
+        """
+        if len(self._low_quality) == 0:
+            # ensure low quality SNPs, if any, are identified
+            self.identify_low_quality_snps()
+
+        return self._snps.loc[self._low_quality]
+
     @property
     def discrepant_merge_positions(self):
         """SNPs with discrepant positions discovered while merging SNPs.
@@ -487,6 +532,59 @@ class SNPs:
         """
         return self._phased
 
+    @property
+    def cluster(self):
+        """Detected chip cluster, if any, per
+        :meth:`compute_cluster_overlap <snps.snps.SNPs.compute_cluster_overlap>`.
+
+        Notes
+        -----
+        Refer to :meth:`compute_cluster_overlap <snps.snps.SNPs.compute_cluster_overlap>`
+        for more details about chip clusters.
+
+        Returns
+        -------
+        str
+            detected chip cluster, e.g., 'c1', else empty str
+        """
+        if not self._cluster:
+            self.compute_cluster_overlap()
+        return self._cluster
+
+    @property
+    def chip(self):
+        """Detected deduced genotype / chip array, if any, per
+        :meth:`compute_cluster_overlap <snps.snps.SNPs.compute_cluster_overlap>`.
+
+        Returns
+        -------
+        str
+            detected chip array, else empty str
+        """
+        if not self._chip:
+            self.compute_cluster_overlap()
+        return self._chip
+
+    @property
+    def chip_version(self):
+        """Detected genotype / chip array version, if any, per
+        :meth:`compute_cluster_overlap <snps.snps.SNPs.compute_cluster_overlap>`.
+
+        Notes
+        -----
+        Chip array version is only applicable to 23andMe (v3, v4, v5)  and AncestryDNA
+        (v1, v2) files.
+
+        Returns
+        -------
+        str
+            detected chip array version, e.g., 'v4', else empty str
+        """
+
+        if not self._chip_version:
+            self.compute_cluster_overlap()
+        return self._chip_version
+
     def heterozygous(self, chrom=""):
         """Get heterozygous SNPs.
 
@@ -585,28 +683,42 @@ class SNPs:
             return True
 
     def save(
-        self, filename="", vcf=False, atomic=True, vcf_alt_unavailable=".", **kwargs
+        self,
+        filename="",
+        vcf=False,
+        atomic=True,
+        vcf_alt_unavailable=".",
+        vcf_qc_only=False,
+        vcf_qc_filter=False,
+        **kwargs,
+    ):
+        warnings.warn(
+            "Method `save` has been replaced by `to_csv`, `to_tsv`, and `to_vcf`.",
+            DeprecationWarning,
+        )
+        return self._save(
+            filename,
+            vcf,
+            atomic,
+            vcf_alt_unavailable,
+            vcf_qc_only,
+            vcf_qc_filter,
+            **kwargs,
+        )
+
+    def _save(
+        self,
+        filename="",
+        vcf=False,
+        atomic=True,
+        vcf_alt_unavailable=".",
+        vcf_chrom_prefix="",
+        vcf_qc_only=False,
+        vcf_qc_filter=False,
+        **kwargs,
     ):
         """Save SNPs to file.
 
-        Parameters
-        ----------
-        filename : str or buffer
-            filename for file to save or buffer to write to
-        vcf : bool
-            flag to save file as VCF
-        atomic : bool
-            atomically write output to a file on local filesystem
-        vcf_alt_unavailable : str
-            representation of VCF ALT allele when ALT is not able to be determined
-        **kwargs
-            additional parameters to `pandas.DataFrame.to_csv`
-
-        Returns
-        -------
-        str
-            path to file in output directory if SNPs were saved, else empty str
-
         References
         ----------
         1. Fluent Python by Luciano Ramalho (O'Reilly). Copyright 2015 Luciano Ramalho,
@@ -615,14 +727,18 @@ class SNPs:
         if "sep" not in kwargs:
             kwargs["sep"] = "\t"
 
-        path, *extra = Writer.write_file(
+        w = Writer(
             snps=self,
             filename=filename,
             vcf=vcf,
             atomic=atomic,
             vcf_alt_unavailable=vcf_alt_unavailable,
+            vcf_chrom_prefix=vcf_chrom_prefix,
+            vcf_qc_only=vcf_qc_only,
+            vcf_qc_filter=vcf_qc_filter,
             **kwargs,
         )
+        path, *extra = w.write()
 
         if len(extra) == 1 and not extra[0].empty:
             self._discrepant_vcf_position = extra[0]
@@ -633,11 +749,112 @@ class SNPs:
 
         return path
 
+    def to_csv(self, filename="", atomic=True, **kwargs):
+        """Output SNPs as comma-separated values.
+
+        Parameters
+        ----------
+        filename : str or buffer
+            filename for file to save or buffer to write to
+        atomic : bool
+            atomically write output to a file on local filesystem
+        **kwargs
+            additional parameters to `pandas.DataFrame.to_csv`
+
+        Returns
+        -------
+        str
+            path to file in output directory if SNPs were saved, else empty str
+        """
+        kwargs["sep"] = ","
+        return self._save(filename=filename, atomic=atomic, **kwargs)
+
+    def to_tsv(self, filename="", atomic=True, **kwargs):
+        """Output SNPs as tab-separated values.
+
+        Note that this results in the same default output as `save`.
+
+        Parameters
+        ----------
+        filename : str or buffer
+            filename for file to save or buffer to write to
+        atomic : bool
+            atomically write output to a file on local filesystem
+        **kwargs
+            additional parameters to `pandas.DataFrame.to_csv`
+
+        Returns
+        -------
+        str
+            path to file in output directory if SNPs were saved, else empty str
+        """
+        kwargs["sep"] = "\t"
+        return self._save(filename=filename, atomic=atomic, **kwargs)
+
+    def to_vcf(
+        self,
+        filename="",
+        atomic=True,
+        alt_unavailable=".",
+        chrom_prefix="",
+        qc_only=False,
+        qc_filter=False,
+        **kwargs,
+    ):
+        """Output SNPs as Variant Call Format.
+
+        Parameters
+        ----------
+        filename : str or buffer
+            filename for file to save or buffer to write to
+        atomic : bool
+            atomically write output to a file on local filesystem
+        alt_unavailable : str
+            representation of ALT allele when ALT is not able to be determined
+        chrom_prefix : str
+            prefix for chromosomes in VCF CHROM column
+        qc_only : bool
+            output only SNPs that pass quality control
+        qc_filter : bool
+            populate FILTER column based on quality control results
+        **kwargs
+            additional parameters to `pandas.DataFrame.to_csv`
+
+        Returns
+        -------
+        str
+            path to file in output directory if SNPs were saved, else empty str
+
+        Notes
+        -----
+        Parameters `qc_only` and `qc_filter`, if true, will identify low quality SNPs per
+        :meth:`identify_low_quality_snps() <snps.snps.SNPs.identify_low_quality_snps>`,
+        if not done already. Moreover, these parameters have no effect if this SNPs
+        object does not map to a cluster per
+        :meth:`compute_cluster_overlap() <snps.snps.SNPs.compute_cluster_overlap>`.
+
+        References
+        ----------
+        1. The Variant Call Format (VCF) Version 4.2 Specification, 8 Mar 2019,
+           https://samtools.github.io/hts-specs/VCFv4.2.pdf
+        """
+        return self._save(
+            filename=filename,
+            vcf=True,
+            atomic=atomic,
+            vcf_alt_unavailable=alt_unavailable,
+            vcf_chrom_prefix=chrom_prefix,
+            vcf_qc_only=qc_only,
+            vcf_qc_filter=qc_filter,
+            **kwargs,
+        )
+
     def _filter(self, chrom=""):
         return self.snps.loc[self.snps.chrom == chrom] if chrom else self.snps
 
     def _read_raw_data(self, file, only_detect_source, rsids):
-        return Reader.read_file(file, only_detect_source, self._resources, rsids)
+        r = Reader(file, only_detect_source, self._resources, rsids)
+        return r.read()
 
     def _assign_par_snps(self):
         """Assign PAR SNPs to the X or Y chromosome using SNP position.
@@ -1523,8 +1740,11 @@ class SNPs:
         return self.remap(target_assembly, complement_bases)
 
     def save_snps(self, filename="", vcf=False, atomic=True, **kwargs):
-        warnings.warn("This method has been renamed to `save`.", DeprecationWarning)
-        return self.save(filename, vcf, atomic, **kwargs)
+        warnings.warn(
+            "Method `save_snps` has been replaced by `to_csv`, `to_tsv`, and `to_vcf`.",
+            DeprecationWarning,
+        )
+        return self._save(filename, vcf, atomic, **kwargs)
 
     @property
     def snp_count(self):
@@ -1730,3 +1950,149 @@ class SNPs:
         d["ezancestry_df"] = predictions
 
         return d
+
+    def compute_cluster_overlap(self, cluster_overlap_threshold=0.95):
+        """Compute overlap with chip clusters.
+
+        Chip clusters, which are defined in [1]_, are associated with deduced genotype /
+        chip arrays and DTC companies.
+
+        This method also sets the values returned by the `cluster`, `chip`, and
+        `chip_version` properties, based on max overlap, if the specified threshold is
+        satisfied.
+
+        Parameters
+        ----------
+        cluster_overlap_threshold : float
+            threshold for cluster to overlap this SNPs object, and vice versa, to set
+            values returned by the `cluster`, `chip`, and `chip_version` properties
+
+        Returns
+        -------
+        pandas.DataFrame
+            pandas.DataFrame with the following columns:
+
+            `company_composition`
+              DTC company composition of associated cluster from [1]_
+            `chip_base_deduced`
+              deduced genotype / chip array of associated cluster from [1]_
+            `snps_in_cluster`
+              count of SNPs in cluster
+            `snps_in_common`
+              count of SNPs in common with cluster (inner merge with cluster)
+            `overlap_with_cluster`
+              percentage overlap of `snps_in_common` with cluster
+            `overlap_with_self`
+              percentage overlap of `snps_in_common` with this SNPs object
+
+        References
+        ----------
+        .. [1] Chang Lu, Bastian Greshake Tzovaras, Julian Gough, A survey of
+               direct-to-consumer genotype data, and quality control tool
+               (GenomePrep) for research, Computational and Structural
+               Biotechnology Journal, Volume 19, 2021, Pages 3747-3754, ISSN
+               2001-0370, https://doi.org/10.1016/j.csbj.2021.06.040.
+        """
+
+        # information from Lu et. al (Ref. [1]_), Table 2 and Fig. 2
+        df = pd.DataFrame(
+            data={
+                "cluster_id": ["c1", "c3", "c4", "c5", "v5"],
+                "company_composition": [
+                    "23andMe-v4",
+                    "AncestryDNA-v1, FTDNA, MyHeritage",
+                    "23andMe-v3",
+                    "AncestryDNA-v2",
+                    "23andMe-v5, LivingDNA",
+                ],
+                "chip_base_deduced": [
+                    "HTS iSelect HD",
+                    "OmniExpress",
+                    "OmniExpress plus",
+                    "OmniExpress plus",
+                    "Illumina GSAs",
+                ],
+                "snps_in_cluster": [0] * 5,
+                "snps_in_common": [0] * 5,
+            }
+        )
+        df.set_index("cluster_id", inplace=True)
+
+        if self.build != 37:
+            to_remap = copy.deepcopy(self)
+            to_remap.remap(37)  # clusters are relative to Build 37
+            self_snps = to_remap.snps[["chrom", "pos"]].drop_duplicates()
+        else:
+            self_snps = self.snps[["chrom", "pos"]].drop_duplicates()
+
+        chip_clusters = self._resources.get_chip_clusters()
+
+        for cluster in df.index.values:
+            cluster_snps = chip_clusters.loc[
+                chip_clusters.clusters.str.contains(cluster)
+            ][["chrom", "pos"]]
+            df.loc[cluster, "snps_in_cluster"] = len(cluster_snps)
+            df.loc[cluster, "snps_in_common"] = len(
+                self_snps.merge(cluster_snps, how="inner")
+            )
+
+        df["overlap_with_cluster"] = df.snps_in_common / df.snps_in_cluster
+        df["overlap_with_self"] = df.snps_in_common / len(self_snps)
+
+        max_overlap = df.overlap_with_cluster.idxmax()
+
+        if (
+            df.overlap_with_cluster.loc[max_overlap] > cluster_overlap_threshold
+            and df.overlap_with_self.loc[max_overlap] > cluster_overlap_threshold
+        ):
+            self._cluster = max_overlap
+            self._chip = df.chip_base_deduced.loc[max_overlap]
+
+            company_composition = df.company_composition.loc[max_overlap]
+
+            if self.source in company_composition:
+                if self.source == "23andMe" or self.source == "AncestryDNA":
+                    i = company_composition.find("v")
+                    self._chip_version = company_composition[i : i + 2]
+            else:
+                logger.warning(
+                    "Detected SNPs data source not found in cluster's company composition"
+                )
+
+        return df
+
+    def identify_low_quality_snps(self):
+        """Identify low quality SNPs based on chip clusters.
+
+        Any low quality SNPs are removed from the
+        :meth:`snps_qc <snps.snps.SNPs.snps_qc>` dataframe and are made
+        available as :meth:`low_quality <snps.snps.SNPs.low_quality>`.
+
+        Notes
+        -----
+        Chip clusters, which are defined in [1]_, are associated with low quality SNPs.
+        As such, low quality SNPs will only be identified when this SNPs object corresponds
+        to a cluster per
+        :meth:`compute_cluster_overlap() <snps.snps.SNPs.compute_cluster_overlap>`.
+        """
+        if self.build != 37:
+            to_remap = copy.deepcopy(self)
+            to_remap.remap(37)  # clusters are relative to Build 37
+            self_snps = to_remap._snps[["chrom", "pos"]]
+        else:
+            self_snps = self._snps[["chrom", "pos"]]
+
+        low_quality_snps = self._resources.get_low_quality_snps()
+
+        if self.cluster:
+            cluster_snps = low_quality_snps.loc[
+                low_quality_snps.cluster.str.contains(self.cluster)
+            ][["chrom", "pos"]]
+            # keep index after merge; https://stackoverflow.com/a/11982843
+            merged = (
+                self_snps.reset_index()
+                .merge(cluster_snps, how="inner")
+                .set_index("rsid")
+            )
+
+            self._low_quality = merged.index
