@@ -33,6 +33,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 import importlib.util
 import io
+import logging
 import os
 import sys
 import tempfile
@@ -361,28 +362,28 @@ class TestSnps(BaseSNPsTestCase):
     def test_save_buffer(self):
         s = SNPs("tests/input/generic.csv")
         out = io.StringIO()
-        s.save(out)
+        s.to_tsv(out)
         self.assertTrue(out.read().startswith("# Generated by snps"))
 
     def test_save_buffer_binary(self):
         s = SNPs("tests/input/generic.csv")
         out = io.BytesIO()
-        s.save(out)
+        s.to_tsv(out)
         self.assertTrue(out.read().startswith(b"# Generated by snps"))
 
     def test_save_no_snps(self):
         s = SNPs()
-        self.assertFalse(s.save())
+        self.assertFalse(s.to_tsv())
 
     def test_save_no_snps_vcf(self):
         s = SNPs()
-        self.assertFalse(s.save(vcf=True))
+        self.assertFalse(s.to_vcf())
 
     def test_save_source(self):
         with tempfile.TemporaryDirectory() as tmpdir:
             s = SNPs("tests/input/GRCh38.csv", output_dir=tmpdir)
             dest = os.path.join(tmpdir, "generic_GRCh38.txt")
-            self.assertEqual(s.save(), dest)
+            self.assertEqual(s.to_tsv(), dest)
             snps = SNPs(dest)
             self.assertEqual(snps.build, 38)
             self.assertTrue(snps.build_detected)
@@ -531,6 +532,166 @@ class TestSnps(BaseSNPsTestCase):
         for snps in self.empty_snps():
             self.assertDictEqual(snps.predict_ancestry(), {})
 
+    def _get_chip_clusters(self, pos=tuple(range(101, 109)), cluster="c1", length=8):
+        df = pd.DataFrame(
+            {"chrom": ["1"] * length, "pos": pos, "clusters": [cluster] * length},
+            columns=["chrom", "pos", "clusters"],
+        )
+        df.chrom = df.chrom.astype(pd.CategoricalDtype(ordered=False))
+        df.pos = df.pos.astype(np.uint32)
+        df.clusters = df.clusters.astype(pd.CategoricalDtype(ordered=False))
+        return df
+
+    def run_cluster_test(self, f, chip_clusters):
+        mock = Mock(return_value=chip_clusters)
+        with patch("snps.resources.Resources.get_chip_clusters", mock):
+            f()
+
+    def test_cluster(self):
+        def f():
+            s = SNPs("tests/input/23andme.txt")
+            self.assertEqual(s.cluster, "c1")
+
+        self.run_cluster_test(f, self._get_chip_clusters())
+
+    def test_chip(self):
+        def f():
+            s = SNPs("tests/input/23andme.txt")
+            self.assertEqual(s.chip, "HTS iSelect HD")
+
+        self.run_cluster_test(f, self._get_chip_clusters())
+
+    def test_chip_version(self):
+        def f():
+            s = SNPs("tests/input/23andme.txt")
+            self.assertEqual(s.chip_version, "v4")
+
+        self.run_cluster_test(f, self._get_chip_clusters())
+
+    def test_chip_version_na(self):
+        def f():
+            s = SNPs("tests/input/myheritage.csv")
+            self.assertEqual(s.cluster, "c3")
+            self.assertEqual(s.chip_version, "")
+
+        self.run_cluster_test(f, self._get_chip_clusters(cluster="c3"))
+
+    def test_compute_cluster_overlap_set_property_values(self):
+        def f():
+            s = SNPs("tests/input/23andme.txt")
+            s.compute_cluster_overlap()
+            self.assertEqual(s.cluster, "c1")
+            self.assertEqual(s.chip, "HTS iSelect HD")
+            self.assertEqual(s.chip_version, "v4")
+
+        self.run_cluster_test(f, self._get_chip_clusters())
+
+    def test_compute_cluster_overlap_threshold_not_met(self):
+        def f():
+            s = SNPs("tests/input/23andme.txt")
+            self.assertEqual(s.cluster, "")
+
+        self.run_cluster_test(f, self._get_chip_clusters(pos=tuple(range(104, 112))))
+
+    def test_compute_cluster_overlap_source_warning(self):
+        def f():
+            s = SNPs("tests/input/generic.csv")
+            self.assertEqual(s.cluster, "c1")
+
+        with self.assertLogs(level=logging.WARN) as cm:
+            self.run_cluster_test(f, self._get_chip_clusters())
+
+        self.assertIn(
+            "Detected SNPs data source not found in cluster's company composition",
+            cm.output[0],
+        )
+
+    def test_compute_cluster_overlap_remap(self):
+        def f():
+            s = SNPs("tests/input/23andme.txt")
+            # drop SNPs not currently remapped by test mapping data
+            s._snps.drop(["rs4", "rs5", "rs6", "rs7", "rs8"], inplace=True)
+            s._build = 36  # manually set build 36
+            self.assertEqual(s.cluster, "c1")
+            self.assertEqual(s.build, 36)  # ensure copy gets remapped
+
+        mock = Mock(
+            return_value=self._get_test_assembly_mapping_data(
+                "NCBI36",
+                "GRCh37",
+                [1] * 8,
+                [101, 101, 102, 102, 103, 103, 0, 0],
+            )
+        )
+        with patch("snps.resources.Resources.get_assembly_mapping_data", mock):
+            self.run_cluster_test(
+                f, self._get_chip_clusters(pos=tuple(range(101, 104)), length=3)
+            )
+
+    def test_snps_qc(self):
+        def f():
+            s = SNPs("tests/input/generic.csv")
+            # identify quality controlled SNPs
+            pd.testing.assert_frame_equal(
+                s.snps_qc, self.generic_snps().drop(["rs4", "rs6"])
+            )
+            # return already identified quality controlled SNPs (test branch)
+            pd.testing.assert_frame_equal(
+                s.snps_qc, self.generic_snps().drop(["rs4", "rs6"])
+            )
+
+        self.run_low_quality_snps_test(f, self.get_low_quality_snps())
+
+    def test_low_quality(self):
+        def f():
+            s = SNPs("tests/input/generic.csv")
+            # identify low quality SNPs
+            pd.testing.assert_frame_equal(
+                s.low_quality, self.generic_snps().loc[["rs4", "rs6"]]
+            )
+            # return already identified low quality SNPs (test branch)
+            pd.testing.assert_frame_equal(
+                s.low_quality, self.generic_snps().loc[["rs4", "rs6"]]
+            )
+
+        self.run_low_quality_snps_test(f, self.get_low_quality_snps())
+
+    def test_snps_qc_low_quality_no_cluster(self):
+        def f():
+            s = SNPs("tests/input/generic.csv")
+            pd.testing.assert_frame_equal(s.snps_qc, self.generic_snps())
+            self.assertEqual(len(s.low_quality), 0)
+
+        self.run_low_quality_snps_test(f, self.get_low_quality_snps(), cluster="")
+
+    def test_identify_low_quality_snps_remap(self):
+        def f():
+            s = SNPs("tests/input/generic.csv")
+            # drop SNPs not currently remapped by test mapping data
+            s._snps.drop(["rs4", "rs5", "rs6", "rs7", "rs8"], inplace=True)
+            s._build = 36  # manually set build 36
+            s.identify_low_quality_snps()
+            pd.testing.assert_frame_equal(
+                s.snps_qc, self.generic_snps().loc[["rs1", "rs3"]]
+            )
+            pd.testing.assert_frame_equal(
+                s.low_quality, self.generic_snps().loc[["rs2"]]
+            )
+            self.assertEqual(s.build, 36)  # ensure copy gets remapped
+
+        mock = Mock(
+            return_value=self._get_test_assembly_mapping_data(
+                "NCBI36",
+                "GRCh37",
+                [1] * 8,
+                [101, 101, 102, 102, 103, 103, 0, 0],
+            )
+        )
+        with patch("snps.resources.Resources.get_assembly_mapping_data", mock):
+            self.run_low_quality_snps_test(
+                f, self.get_low_quality_snps(pos=(102, 1001))
+            )
+
 
 class TestSNPsMerge(TestSnps):
     def assert_results(self, results, expected_results):
@@ -585,7 +746,7 @@ class TestSNPsMerge(TestSnps):
             self.assertEqual(s.source, "generic, 23andMe")
             self.assertListEqual(s._source, ["generic", "23andMe"])
             dest = os.path.join(tmpdir, "generic__23andMe_GRCh37.txt")
-            self.assertEqual(s.save(), dest)
+            self.assertEqual(s.to_tsv(), dest)
             s = SNPs(dest)
             self.assertEqual(s.source, "generic, 23andMe")
             self.assertListEqual(s._source, ["generic", "23andMe"])
@@ -962,7 +1123,25 @@ class TestDeprecatedMethods(TestSnps):
                 )
                 self.run_parsing_tests(dest, "generic")
 
-        self.run_deprecated_test(f, "This method has been renamed to `save`.")
+        self.run_deprecated_test(
+            f,
+            "Method `save_snps` has been replaced by `to_csv`, `to_tsv`, and `to_vcf`.",
+        )
+
+    def test_save(self):
+        def f():
+            with tempfile.TemporaryDirectory() as tmpdir:
+                snps = SNPs("tests/input/generic.csv", output_dir=tmpdir)
+                dest = os.path.join(tmpdir, "generic_GRCh37.csv")
+                self.assertEqual(
+                    snps.save(sep=","),
+                    dest,
+                )
+                self.run_parsing_tests(dest, "generic")
+
+        self.run_deprecated_test(
+            f, "Method `save` has been replaced by `to_csv`, `to_tsv`, and `to_vcf`."
+        )
 
     def test_snp_count(self):
         def f():
